/* GGL - GPUL Game Library
 * Copyright (C) 1998 G.P.U.L. (Group of programmers and users of Linux)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* Tipo de dato GglSpriteAnim */
/* Contiene cada animacion de un sprite */
#ifndef __GGLANIM_H__
#define __GGLANIM_H__

#include <glib.h>
#include <ggl/gglbase.h>
#include <ggl/gglframe.h>
#include <ggl/ggldescriptor.h>

#define GGL_ANIM(anim) (GGL_CAST(anim,GglAnim*,GGLANIM))

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef struct _GglAnim GglAnim;

struct _GglAnim{
  GglBase base;
  int id;
  int	speed;		/* The speed in frames per second (fps)*/
  GList *frames;	/* List of frames */
  GglDescriptor* cur_frame;	/* Current Frame */
  int num_frames;	/* The number of frames */
};

/* Constructors */
GglAnim*	ggl_anim_new		(void);
void		ggl_anim_init		(GglAnim *anim);

void		ggl_anim_dispose	(GglAnim *anim);

void		ggl_anim_finalize	(GglAnim *anim);

void		ggl_anim_append		(GglAnim *anim,
					 GglFrame *frame);
void		ggl_anim_prepend	(GglAnim *anim,
					 GglFrame *frame);
void		ggl_anim_remove		(GglAnim *anim,
					 GglFrame *frame);
void		ggl_anim_insert		(GglAnim *anim,
					 GglFrame *frame,
					 int position);

/* Clone creates a new animatin, with his own frame position,
but with the same frames (Sharing this data you can eat up
less memory) */
GglAnim*	ggl_anim_clone		(GglAnim *anim);

/* Main method for drawing an anim */
void		ggl_anim_draw	        (GglAnim *anim,
					 GglVisual *visual,
					 int x,
					 int y);
/* Other methods */				 
void		ggl_anim_set_next_frame	(GglAnim *anim);
					
void		ggl_anim_set_frame	(GglAnim *anim,
					 GglDescriptor *descriptor);
void		ggl_anim_set_frame_nth	(GglAnim *anim,
					 int frame);

GglDescriptor*	ggl_anim_get_frame	(GglAnim *anim);

void		ggl_anim_draw_frame	(GglAnim *anim,
					 int frame,
					 GglVisual *visual,
					 int x,
					 int y);

int		ggl_anim_length 	(GglAnim *anim);

void		ggl_anim_get_frame_size	(GglAnim *anim,
					 int frame,
					 int *width,
					 int *height);
/* Current frame */
void		ggl_anim_cur_frame_size	(GglAnim *anim,
					 int *width,
					 int *height);
void		ggl_anim_cur_frame_hook	(GglAnim *anim,
					 short *x,
					 short *y);

#ifdef __cplusplus
      }
#endif /* __cplusplus */

#endif /* __GGLANIM_H__ */

