/* GGL - GPUL Game Library
 * Copyright (C) 1998 G.P.U.L. (Group of programmers and users of Linux)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef __GGLSCREEN_H__
#define __GGLSCREEN_H__

#include <glib.h>
#include <ggl/gglvisual.h>
#include <ggl/gglrectangle.h>
#include <ggl/gglbase.h>
#include <ggl/ggltimeout.h>
#include <ggl/ggllayer.h>

#define GGL_SCREEN(screen) (GGL_CAST(screen,GglScreen*,GGLSCREEN))

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef struct _GglScreen GglScreen;

struct _GglScreen{
  GglBase base;
  int id;
  GglTimeout *timer;  /* Timer for updating the screen */
  GList *layers;      /* Screen layer list */
  GglVisual *buffer;  /* Visual containing the buffer for drawing */
  GglVisual *screen;  /* Visual containing the whole screen (if there isn't double-buffering)*/
  int page_flipping;  /* 1=page_flipping, 0=memory buffer */
};

GglScreen*	ggl_screen_new			(int width,
						 int height,
						 int vwidth,
						 int vheight,
                                                 GglDepth depth,
                                                 char *display_type);

GglScreen*	ggl_screen_new_onvisual		(GglVisual *visual,
						 int width,
						 int height,
						 int vwidth,
						 int vheight,
                                                 GglDepth depth);

void		ggl_screen_init			(GglScreen *screen,
						 int width,
						 int height,
						 int vwidth,
						 int vheight,
                                                 GglDepth depth,
                                                 char *display_type);

void		ggl_screen_init_onvisual	(GglScreen *screen,
						 GglVisual *visual,
						 int width,
						 int height,
						 int vwidth,
						 int vheight,
                                                 GglDepth depth);

void		ggl_screen_dispose		(GglScreen *screen);

void		ggl_screen_finalize		(GglScreen *screen);

void		ggl_screen_add_layer	        (GglScreen *screen,
						 GglLayer *layer);

void		ggl_screen_remove_layer	        (GglScreen *screen,
						 GglLayer *layer);

int		ggl_screen_get_layers	        (GglScreen *screen);

GglLayer*	ggl_screen_get_layer_nth        (GglScreen *screen,
						 int layer);

int		ggl_screen_get_page_flipping	(GglScreen *screen);

void		ggl_screen_show			(GglScreen *screen,
						 int fps);

GglVisual*      ggl_screen_get_phys_visual      (GglScreen *screen);

GglTimeout*     ggl_screen_get_timeout          (GglScreen *screen);
    
#ifdef __cplusplus
      }
#endif /* __cplusplus */

#endif /* __GGLSCREEN_H__ */

