/* GGL - GPUL Game Library
 * Copyright (C) 1998 G.P.U.L. (Group of programmers and users of Linux)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef __GGLSPRITE_H__
#define __GGLSPRITE_H__

#include <glib.h>
#include <ggl/gglobserver.h>
#include <ggl/gglanim.h>
#include <ggl/ggldescriptor.h>
#include <ggl/gglgraphic.h>

#define GGL_SPRITE(sprite) (GGL_CAST (sprite, GglSprite*, GGLSPRITE))

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef struct _GglSprite GglSprite;

struct _GglSprite{
  GglGraphic graphic;
  int id;
  GList *anims;		/* Anim list */
  float speed_x,speed_y;/* Speed */
  GglDescriptor *cur_anim;/* Current anim */
};


/* Constructors */
GglSprite*	ggl_sprite_new		(void);

void		ggl_sprite_init		(GglSprite *sprite);

void		ggl_sprite_dispose	(GglSprite *sprite);

void		ggl_sprite_finalize	(GglSprite *sprite);

void		ggl_sprite_append	(GglSprite *sprite,
					 GglAnim *anim);
void		ggl_sprite_prepend	(GglSprite *sprite,
					 GglAnim *anim);
void		ggl_sprite_remove	(GglSprite *sprite,
					 GglAnim *anim);
void		ggl_sprite_insert	(GglSprite *sprite,
					 GglAnim *anim,
					 int position);
int 		ggl_sprite_set_anim	(GglSprite *sprite,
					 GglAnim* anim);
void 		ggl_sprite_set_anim_nth	(GglSprite *sprite,
					 int anim);
GglDescriptor* 	ggl_sprite_get_anim	(GglSprite *sprite);
void 		ggl_sprite_set_frame	(GglSprite *sprite,
					 GglDescriptor* descriptor);
void 		ggl_sprite_set_frame_nth(GglSprite *sprite,
					 int frame);
GglDescriptor* 	ggl_sprite_get_frame	(GglSprite *sprite);

/* clone creates a new sprite sharing the same animations and frames,
   eating up less memory */
GglSprite*	ggl_sprite_clone	(GglSprite *sprite);

/* Other methods */
void		ggl_sprite_move		(GglSprite *sprite,
					 float x,
					 float y);

void		ggl_sprite_position	(GglSprite *sprite,
					 float *x,
					 float *y);

void		ggl_sprite_set_speed	(GglSprite *sprite,
					 float sx,
					 float sy);

void		ggl_sprite_get_speed	(GglSprite *sprite,
					 float *sx,
					 float *sy);

int 		ggl_sprite_collision	(GglSprite *first,
					 GglSprite *second,
					 float *angle);
void		ggl_sprite_bound	(GglSprite *sprite, 
					 float angle);

#ifdef __cplusplus
      }
#endif /* __cplusplus */

#endif /* __GGLSPRITE_H__ */

