/* GGL - GPUL Game Library
 * Copyright (C) 1998 G.P.U.L. (Group of programmers and users of Linux)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <ggl/ggldevice.h>
#include <stdlib.h>

void ggl_device_init(GglDevice *device)
{
    device-> id=GGLDEVICE;
    device-> observers=NULL;
    device-> ticker=ggl_do_nothing;
    device-> timeout=NULL;
}


GglDevice* ggl_device_new()
{
    GglDevice *device=(GglDevice *)malloc(sizeof(GglDevice));
    ggl_base_set_size(GGL_BASE(device),sizeof(GglDevice));
    ggl_device_init(device);
    
    return device;
}

void ggl_device_finalize(GglDevice *device)
{
    g_list_free(device->observers);
    /* FIXME: add timer code here */
}

void ggl_device_dispose(GglDevice *device)
{
    ggl_device_finalize(device);
    free(device);
}

void ggl_device_attach(GglDevice *device, GglObserver *observer)
{
    GglDeviceItem *obs=(GglDeviceItem *)malloc(sizeof(GglDeviceItem));
    obs->observer=observer;
    obs->mask=GGL_EVMASK_ALL;
    device->observers=g_list_append(device->observers,obs);
}

int ggldevice_find(GglDevice *device, GglObserver *observer)
{
    int i;
    for (i=1;i<g_list_length(device->observers);i++)
    {
	if ( ((GglDeviceItem *)g_list_nth(device->observers, i))->observer==observer)
	    return i;
    }
    return -1;
}

int ggl_device_detach(GglDevice *device, GglObserver *observer)
{
    int position;

    if ((position=ggldevice_find(device,observer))!=-1)
    {
	device->observers=g_list_remove(device->observers,g_list_nth(device->observers,position));
	return 0;
    }
    else return -1;

}
int ggl_device_set_mask (GglDevice *device, GglObserver *observer, GglEventMask mask)
{
    int position;
    GglDeviceItem *item;

    if ((position=ggldevice_find(device,observer))!=-1)
    {
	item=(GglDeviceItem *)g_list_nth(device->observers,position);
	item->mask=mask;
	return 0;
    }
    else return -1;
}


void ggldevice_tick(GglDevice *device)
{
    device->ticker(device);
}

void ggl_device_install (GglDevice *device, GglTimeout *timeout)
{
    GglCallback cb;
    cb.callback=(GglFunc)ggldevice_tick;
    cb.data=device;
    
    ggl_timeout_add_callback(timeout,cb);
}

void ggl_device_foreach_observer (GglDevice *device, GglDevCallBack callback,
				  void *data)
{
    g_list_foreach(device->observers, (GFunc)callback, data);
}
