/* GGL - GPUL Game Library
 * Copyright (C) 1998 G.P.U.L. (Group of programmers and users of Linux)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <ggl/ggljoydev.h>
#include <stdlib.h>

void ggljoydev_ticker(GglJoyDev *joydev);

void ggl_joydev_init(GglJoyDev *joydev, GglVisual *visual)
{
    gii_input_t input; // Joystick input type must be obtained explicitly
    ggl_device_init(GGL_DEVICE(joydev));
    joydev-> id=GGLJOYDEV;
    joydev-> visual=visual;
    input=giiOpen("linux-joy",NULL);
    if (!input)
    {
	g_warning("ggl_joydev_init: Can't open joystick device\n");
	return;
    }
    ggiJoinInputs(visual->visual, input);
    GGL_DEVICE(joydev)->ticker=(void(*)(GglDevice*))ggljoydev_ticker;
}

GglJoyDev* ggl_joydev_new(GglVisual *visual)
{
    GglJoyDev *joydev=(GglJoyDev *)malloc(sizeof(GglJoyDev));
    ggl_base_set_size(GGL_BASE(joydev),sizeof(GglJoyDev));
    
    ggl_joydev_init(joydev,visual);
    
    return joydev;
}

void ggl_joydev_finalize(GglJoyDev *joydev)
{
    ggl_device_finalize(GGL_DEVICE(joydev));
    ggiRemoveEventMask(joydev->visual, emPointer);
}

void ggl_joydev_dispose(GglJoyDev *joydev)
{
    ggl_joydev_finalize(joydev);
    free(joydev);
}

void ggljoydev_button_notify(GglDeviceItem *devitem, ggi_event *event)
{
    if (devitem == NULL) g_warning("ggljoydev_button_notify: NULL pointer\n");
    switch (event->any.type)
    {
    case evValAbsolute:
	if (devitem->mask & GGL_EVMASK_AXIS)
	{
	    int first=event->val.first;
	    int count=event->val.count;
	    int i;
	    for (i=0;i<count;i++)
	    {
		ggl_observer_update_axis(devitem->observer,GGL_JOYSTICK,0,
					 first+i,event->val.value[first+i]);
	    }
	}
	break;
    }
}

void ggljoydev_ticker(GglJoyDev *joydev)
{
    GglVisual *visual=joydev->visual;
    ggi_event event;
    struct timeval t;
    int err;
    t.tv_sec=0;
    t.tv_usec=0;
    if ((err=ggiEventPoll(visual->visual,emValuator,&t))>0)
    {
	ggiEventRead(visual->visual,&event,emValuator);
	ggl_device_foreach_observer(GGL_DEVICE(joydev),
				    (GglDevCallBack)ggljoydev_button_notify,
				    &event);
    }else{
	if (err<0)
	    g_warning("GGLJOYDEV: error code <%i>",err);
    }
}
