/* GGL - GPUL Game Library
 * Copyright (C) 1998 G.P.U.L. (Group of programmers and users of Linux)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <ggl/gglkeydev.h>
#include <stdlib.h>

void gglkeydev_ticker(GglKeyDev *keydev);

void ggl_keydev_init(GglKeyDev *keydev, GglVisual *visual)
{
    ggl_device_init(GGL_DEVICE(keydev));
    keydev-> id=GGLKEYDEV;
    keydev-> visual=visual;
//    ggiAddEventMask(keydev->visual,emKey);
    GGL_DEVICE(keydev)->ticker=(void(*)(GglDevice*))gglkeydev_ticker;
}

GglKeyDev* ggl_keydev_new(GglVisual *visual)
{
    GglKeyDev *keydev=(GglKeyDev *)malloc(sizeof(GglKeyDev));
    ggl_base_set_size(GGL_BASE(keydev),sizeof(GglKeyDev));
    
    ggl_keydev_init(keydev,visual);
    
    return keydev;
}

void ggl_keydev_finalize(GglKeyDev *keydev)
{
    ggl_device_finalize(GGL_DEVICE(keydev));
    ggiRemoveEventMask(keydev->visual, emKey);  
}

void ggl_keydev_dispose(GglKeyDev *keydev)
{
    ggl_keydev_finalize(keydev);
    free(keydev);
}

void gglkeydev_button_notify(GglDeviceItem *devitem, ggi_event *event)
{
    if (devitem == NULL) g_warning("gglkeydev_button_notify: NULL pointer\n");
    if (devitem->mask & GGL_EVMASK_BUTTON)
	ggl_observer_update_button(devitem->observer,GGL_KEYBOARD,0,
				   event->key.label,
				   (event->any.type==evKeyPress || event->any.type==evKeyRepeat));
}

void gglkeydev_ticker(GglKeyDev *keydev)
{
    GglVisual *visual=keydev->visual;
    ggi_event event;
    struct timeval t;
    int err;
    t.tv_sec=0;
    t.tv_usec=0;
    if ((err=ggiEventPoll(visual->visual,emKey,&t))>0)
    {
	ggiEventRead(visual->visual,&event,emKey);
	ggl_device_foreach_observer(GGL_DEVICE(keydev),
				    (GglDevCallBack)gglkeydev_button_notify,
				    &event);
    }else{
	if (err<0)
	    g_warning("GGLKEYDEV: error code <%i>",err);
    }
}
