/* GGL - GPUL Game Library
 * Copyright (C) 1998 G.P.U.L. (Group of programmers and users of Linux)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <ggl/gglgc.h>
#include <ggl/gglframe.h>
#include <stdlib.h>
#include <stdio.h>

#ifndef PI
  #define PI 3.14159265359
#endif
void ggl_frame_init(GglFrame *frame, GglVisual *img)
{
    frame-> id=GGLFRAME;
    frame-> img=img;
    frame-> transparent=NULL;
    frame-> width=img->width;
    frame-> height=img->height;
    frame-> hook_x=0;
    frame-> hook_y=0;
    frame-> action_x=0;
    frame-> action_y=0;
}

GglFrame* ggl_frame_new (GglVisual *img)
{
    GglFrame *frame=(GglFrame *)malloc(sizeof(GglFrame));
    ggl_base_set_size(GGL_BASE(frame),sizeof(GglFrame));

    ggl_frame_init(frame,img);

    return frame;
}

void ggl_frame_finalize (GglFrame *frame)
{
}

void ggl_frame_dispose (GglFrame *frame)
{
  free(frame);
}

void ggl_frame_get_hook (GglFrame *frame, short *x, short *y)
{
    *x=frame-> hook_x;
    *y=frame-> hook_y;
}

void ggl_frame_set_hook (GglFrame *frame, short x, short y)
{
    frame-> hook_x=x;
    frame-> hook_y=y;
}

void ggl_frame_get_action (GglFrame *frame, short *x, short *y)
{
    *x=frame-> action_x;
    *y=frame-> action_y;
}

void ggl_frame_set_action (GglFrame *frame, short x, short y)
{
    frame-> action_x=x;
    frame-> action_y=y;
}


void		gglframe_blit	(GglVisual *visual,
					 GglVisual *source,
					 GglMask *mask,
					 int x,
					 int y,
					 int sourcex,
					 int sourcey,
					 int swidth,
                                         int sheight)
{
    int clipx0,clipy0,clipx1,clipy1;
    int i,j,inc;
    int x0,y0,x1,y1;
    int mask_pos=0;
    int state=0;
    int inc_bytes;
    int bits_size=(ggl_visual_get_bits_size(source))>>3; /* 8 bits -> 1 byte */
    void *read_buffer=ggl_visual_get_read_pointer(source);
    void *write_buffer=ggl_visual_get_write_pointer(visual);

    if (source==NULL || visual==NULL || mask==NULL) g_error("ggl_frame_blit: NULL pointers!\n");
    ggl_gc_get_clipping(visual,&clipx0,&clipy0,&clipx1,&clipy1);
    x0=MAX(clipx0,x);
    y0=MAX(clipy0,y);
    x1=MIN(clipx1,x+swidth);
    y1=MIN(clipy1,y+sheight);
    if (swidth==640)
    {
	printf("x0=%i,y0=%i,x1=%i,y1=%i\n",x0,y0,x1,y1);
    }
    read_buffer=read_buffer+(sourcex+(sourcey)*source->width)*bits_size;
    write_buffer=write_buffer+(x+y*visual->width)*bits_size;
    for(j=0;j<y1-y0;j++)
    {
	state=0;
        for(i=0;i<x1-x0;)
        {
            inc=mask->data[mask_pos];
            inc_bytes=inc*bits_size;
            if (state!=0)
	    {
		if (x1>=630)
		    memcpy(write_buffer,read_buffer,inc_bytes);
		else memset(write_buffer,0,inc_bytes);
		state=0;
            }
            else state=1;
            i+=inc;
            read_buffer+=inc_bytes;
            write_buffer+=inc_bytes;
            mask_pos++;
        }
        for(;i<mask->width;i++)
        {
            inc=mask->data[mask_pos];
            i+=inc;
            inc_bytes=inc*bits_size;
            read_buffer+=inc_bytes;
            write_buffer+=inc_bytes;
            mask_pos++;
        }
        write_buffer+=(visual->width - source->width)*bits_size;
    }
    
}

void		gglframe_stretch_blit	(GglVisual *visual,
					 GglVisual *source,
					 GglMask mask,
					 int x,
					 int y,
					 int width,
					 int height,
                                         int sourcex,
                                         int sourcey,
                                         int swidth,
                                         int sheight);


void ggl_frame_set_transparent (GglFrame *frame, GglColor *transp)
{
    frame->transparent = transp;
}

GglColor* ggl_frame_get_transparent (GglFrame *frame)
{
    return frame->transparent;
}

void gglframe_transp_blit(GglVisual *source, int width, int height, GglVisual *target, int tx, int ty, GglColor *transp)
{
    int clipx0,clipy0,clipx1,clipy1;
    int bits_size=(ggl_visual_get_bits_size(source))>>3; /* 8 bits -> 1 byte */
    void *read_buffer=ggl_visual_get_read_pointer(source);
    void *write_buffer=ggl_visual_get_write_pointer(target);
    int i,j;
    int tx1=tx+width;
    int ty1=ty+height;
    int movex=width,movey=height; /* Number of real bytes to move */
    int twidth=target->width; /* More efficient than 2 pointer dereferences */
    ggi_pixel tcolor=transp->color;
    if (source==NULL || target==NULL) g_error("ggl_frame_blit: NULL pointers!\n");
    ggl_gc_get_clipping(target,&clipx0,&clipy0,&clipx1,&clipy1);
    

    if (tx <clipx0) {read_buffer+=(clipx0-tx)*bits_size;movex-=clipx0-tx;tx=clipx0;}
    if (ty <clipy0) {read_buffer+=(clipy0-ty)*width*bits_size;movey-=clipy0-ty;ty=clipy0;}
    if (tx1>clipx1) {movex-=(tx1-clipx1);}
    if (ty1>clipy1) {movey-=(ty1-clipy1);}

    if (movex<=0 || movey<=0) return; /* Nothing to draw */

    switch (bits_size){
    case 1:
	write_buffer+=(tx+ty*twidth);
	for (j=0;j<movey;j++)
	{
	    unsigned char *lectura=read_buffer;
	    unsigned char *escritura=write_buffer;
	    for (i=0;i<movex;i++)
	    {
                if (*lectura!=tcolor)
		    *(escritura)=*(lectura);
		escritura++;
		lectura++;
	    }
	    read_buffer+=width;
	    write_buffer+=twidth;
	}
	break;
    case 2:
	write_buffer+=(tx+ty*twidth) << 1;
	for (j=0;j<movey;j++)
	{
	    unsigned short *lectura=read_buffer;
	    unsigned short *escritura=write_buffer;
	    for (i=0;i<movex;i++)
	    {
                if (*lectura!=tcolor)
		    *(escritura)=*(lectura);
		escritura++;
		lectura++;
	    }
	    read_buffer+=width<<1;
	    write_buffer+=twidth << 1;
	}
	break;
    case 3:
	break;
    case 4:
	write_buffer+=(tx+ty*twidth) << 2;
	for (j=0;j<movey;j++)
	{
	    unsigned *lectura=read_buffer;
	    unsigned *escritura=write_buffer;
	    for (i=0;i<movex;i++)
	    {
                if (*lectura!=tcolor)
		    *(escritura)=*(lectura);
		escritura++;
		lectura++;
	    }
	    read_buffer+=width<<2;
	    write_buffer+=twidth << 2;
	}
	break;
    default:
	g_warning("gglframe_transp_blit: Unknown color depth to blit\n");
	printf("Requested depth: %i\n",bits_size);
    }
}

void ggl_frame_draw (GglFrame *frame, GglVisual *visual, int x, int y)
{
    if (frame->transparent)
    {
/*	gglframe_blit(visual,
		      frame->img,
		      frame->mask,
		      x-frame->hook_x,
		      y-frame->hook_y,
		      0,0,
		      frame->width,
		      frame->height);*/
	gglframe_transp_blit(frame->img,
			     frame->width,
			     frame->height,
			     visual,
			     x-frame->hook_x,
			     y-frame->hook_y,
			     frame->transparent
			    );
    }else{
	ggl_gc_cross_blit(frame->img,
			0,
			0,
			frame->width,
			frame->height,
			visual,
			x-frame->hook_x,
			y-frame->hook_y
		       );
    }
}

#ifndef MAX
  #define MAX(a,b) (a)>(b)?(a):(b)
#endif

#ifndef MIN
  #define MIN(a,b) (a)<(b)?(a):(b)
#endif

void ggl_frame_get_size (GglFrame *frame, int *width, int *height)
{
    *width=frame->width;
    *height=frame->height;
}

char matrix0[3][3]={
    {0,0,0},
    {1,1,1},
    {1,1,1}
};

char matrix0_45_1[3][3]={
    {0,0,1},
    {1,1,1},
    {1,1,1}
};

char matrix0_45_2[3][3]={
    {0,0,0},
    {0,1,1},
    {1,1,1}

};
char matrix45[3][3]={
    {0,0,1},
    {0,1,1},
    {1,1,1}
};

char matrix45_90_1[3][3]={
    {0,1,1},
    {0,1,1},
    {1,1,1}
};

char matrix45_90_2[3][3]={
    {0,0,1},
    {0,1,1},
    {0,1,1}
};

char matrix90[3][3]={
    {0,1,1},
    {0,1,1},
    {0,1,1}
};

char matrix90_135_1[3][3]={
    {1,1,1},
    {0,1,1},
    {0,1,1}
};

char matrix90_135_2[3][3]={
    {0,1,1},
    {0,1,1},
    {0,0,1}
};

char matrix135[3][3]={
    {1,1,1},
    {0,1,1},
    {0,0,1}
};

char matrix135_180_1[3][3]={
    {1,1,1},
    {1,1,1},
    {0,0,1}
};

char matrix135_180_2[3][3]={
    {1,1,1},
    {0,1,1},
    {0,0,0}
};

char matrix180[3][3]={
    {1,1,1},
    {1,1,1},
    {0,0,0}
};

char matrix180_225_1[3][3]={
    {1,1,1},
    {1,1,1},
    {1,0,0}
};

char matrix180_225_2[3][3]={
    {1,1,1},
    {1,1,0},
    {0,0,0}
};

char matrix225[3][3]={
    {1,1,1},
    {1,1,0},
    {1,0,0}
};

char matrix225_270_1[3][3]={
    {1,1,1},
    {1,1,0},
    {1,1,0}
};

char matrix225_270_2[3][3]={
    {1,1,0},
    {1,1,0},
    {1,0,0}
};

char matrix270[3][3]={
    {1,1,0},
    {1,1,0},
    {1,1,0}
};

char matrix270_315_1[3][3]={
    {1,1,0},
    {1,1,0},
    {1,1,1}
};

char matrix270_315_2[3][3]={
    {1,0,0},
    {1,1,0},
    {1,1,0}
};

char matrix315[3][3]={
    {1,0,0},
    {1,1,0},
    {1,1,1}
};

char matrix315_360_1[3][3]={
    {1,0,0},
    {1,1,1},
    {1,1,1}
};

char matrix315_360_2[3][3]={
    {0,0,0},
    {1,1,0},
    {1,1,1}
};

static int matrix_cmp(char a[3][3],char b[3][3])
{
    int i,j;
    for (i=0;i<3;i++)
    {
	for (j=0;j<3;j++)
	{
	    if (a[i][j]!=b[i][j]) return 0;
	}
    }
    return 1;
}

float gglframe_angle(GglFrame *frame, int x, int y)
{
    /* We will check the eight pixels around (x,y), to determine an angle
     * it should provide 16 different angles :) */
    char matrix[3][3];
    float degrees;
    ggi_pixel transp = frame->transparent->color;
    void *read_buffer=ggl_visual_get_read_pointer(frame->img);
    int bits_size=(ggl_visual_get_bits_size(frame->img))>>3; /* 8 bits -> 1 byte */
    int i,j,pos,skip;

    pos=(x - 1 + ((y - 1) * frame->width))*bits_size;
    skip=(frame->width - 3 ) * bits_size;
    
    for (j=0;j<3;j++)
    {
	for (i=0;i<3;i++)
	{
	    int eq=0;
	    switch (bits_size)
	    {/* FIXME: NOT PORTABLE AT ALL !!! */
	    case 1:eq = (*((unsigned char*)(read_buffer+pos)) != transp);break;
	    case 2:eq = (*((unsigned short*)(read_buffer+pos)) != transp);break;
	    case 3:eq = ((*((unsigned*)(read_buffer+pos))&0xFFFFFF) != transp);break;
	    case 4:eq = (*((unsigned*)(read_buffer+pos)) != transp);break;
	    default: g_warning("gglframe_angle: Unsupported color depth\n");
	    }
	    matrix[j][i]=eq;
	    pos+=bits_size;
	}
	pos+=skip;
    }
    if (matrix_cmp(matrix, matrix0)) degrees = 0.0;
    if (matrix_cmp(matrix, matrix0_45_1)) degrees = 22.5;
    if (matrix_cmp(matrix, matrix0_45_2)) degrees = 22.5;
    if (matrix_cmp(matrix, matrix45)) degrees = 45.0;
    if (matrix_cmp(matrix, matrix45_90_1)) degrees = 67.5;
    if (matrix_cmp(matrix, matrix45_90_2)) degrees = 67.5;
    if (matrix_cmp(matrix, matrix90)) degrees = 90.0;
    if (matrix_cmp(matrix, matrix90_135_1)) degrees = 112.5;
    if (matrix_cmp(matrix, matrix90_135_2)) degrees = 112.5;
    if (matrix_cmp(matrix, matrix135)) degrees = 135.0;
    if (matrix_cmp(matrix, matrix135_180_1)) degrees = 157.5;
    if (matrix_cmp(matrix, matrix135_180_2)) degrees = 157.5;
    if (matrix_cmp(matrix, matrix180)) degrees = 180.0;
    if (matrix_cmp(matrix, matrix180_225_1)) degrees = 202.5;
    if (matrix_cmp(matrix, matrix180_225_2)) degrees = 202.5;
    if (matrix_cmp(matrix, matrix225)) degrees = 225.0;
    if (matrix_cmp(matrix, matrix225_270_1)) degrees = 247.5;
    if (matrix_cmp(matrix, matrix225_270_2)) degrees = 247.5;
    if (matrix_cmp(matrix, matrix270)) degrees = 270.0;
    if (matrix_cmp(matrix, matrix270_315_1)) degrees = 292.5;
    if (matrix_cmp(matrix, matrix270_315_2)) degrees = 292.5;
    if (matrix_cmp(matrix, matrix315)) degrees = 315.0;
    if (matrix_cmp(matrix, matrix315_360_1)) degrees = 337.5;
    if (matrix_cmp(matrix, matrix315_360_2)) degrees = 337.5;

    for (j=0;j<3;j++)
    {
	for (i=0;i<3;i++)
	{
	    printf("%i ",matrix[j][i]);
	}
	printf("\n");
    }printf("DEGREES=%f\n",degrees);
    
    return degrees*PI/180;
}

/* Finds the nearest border (because of the point of collision can be inside
 * the sprite, not in the border itself) */
static void search_border(GglFrame *frame,int x, int y, int dx, int dy, int *x1, int *y1)
{
    int incx=(dx>0)?1:-1;
    int incy=(dy << 16)/dx;
    ggi_pixel transp = frame->transparent->color;
    void *read_buffer=ggl_visual_get_read_pointer(frame->img);
    int bits_size=(ggl_visual_get_bits_size(frame->img))>>3; /* 8 bits -> 1 byte */
    int pos,eq;

    y<<=16;
    do{
	x+=incx;
	y+=incy*incx;
	if (x<0 || y<0) break;
	pos=(x+(y>>16)*frame->width)*bits_size;
	switch (bits_size)
	{/* FIXME: NOT PORTABLE AT ALL !!! */
	case 1:eq = (*((unsigned char*)(read_buffer+pos)) == transp);break;
	case 2:eq = (*((unsigned short*)(read_buffer+pos)) == transp);break;
	case 3:eq = ((*((unsigned*)(read_buffer+pos))&0xFFFFFF) == transp);break;
	case 4:eq = (*((unsigned*)(read_buffer+pos)) == transp);break;
	default: g_warning("gglframe_angle: Unsupported color depth\n");
	}
    }while(!eq);
    x-=incx;
    y-=incy*incx;
    *x1=x;
    *y1=y>>16;
}

/* Checks at pixel level if the frame 1 is colliding with the frame 2,
 the areas are used to limit the areas that must be taken into account.
 Of course these areas must be equal in width and height.*/
int gglframe_overlap(GglFrame *frame1, GglFrame *frame2, GglRectangle area1, GglRectangle area2, int dx, int dy, float *angle)
{
    ggi_pixel transp1 = frame1->transparent->color;
    ggi_pixel transp2 = frame2->transparent->color;
    void *read_buffer1=ggl_visual_get_read_pointer(frame1->img);
    void *read_buffer2=ggl_visual_get_read_pointer(frame2->img);
    int bits_size1=(ggl_visual_get_bits_size(frame1->img))>>3; /* 8 bits -> 1 byte */
    int bits_size2=(ggl_visual_get_bits_size(frame2->img))>>3; /* 8 bits -> 1 byte */
    int i,j,pos1,pos2,skip1,skip2;
    
    if (area1.width!=area2.width || area1.height!=area2.height)
    {
	g_warning("gglframe_overlap: Areas are different in size\n");
    }
    pos1=(area1.x + (area1.y * frame1->width))*bits_size1;
    pos2=(area2.x + (area2.y * frame2->width))*bits_size2;
    skip1=(frame1->width-area1.width)*bits_size1;
    skip2=(frame2->width-area2.width)*bits_size2;

    for (j=0; j<area1.height; j++)
    {
	for (i=0; i<area1.width; i++)
	{
	    int eq1;
	    int eq2;
	    switch (bits_size1)
	    {/* FIXME: NOT PORTABLE AT ALL !!! */
	    case 1:eq1 = *((unsigned char*)(read_buffer1+pos1)) != transp1;break;
	    case 2:eq1 = *((unsigned short*)(read_buffer1+pos1)) != transp1;break;
	    case 3:eq1 = (*((unsigned*)(read_buffer1+pos1))&0xFFFFFF) != transp1;break;
	    case 4:eq1 = *((unsigned*)(read_buffer1+pos1)) != transp1;break;
	    default: g_warning("gglframe_overlap: Unsupported color depth\n");
	    }
	    switch (bits_size2)
	    {/* FIXME: NOT PORTABLE AT ALL !!! */
	    case 1:eq2 = *((unsigned char*)(read_buffer2+pos2)) != transp2;break;
	    case 2:eq2 = *((unsigned short*)(read_buffer2+pos2)) != transp2;break;
	    case 3:eq2 = (*((unsigned*)(read_buffer2+pos2))&0xFFFFFF) != transp2;break;
	    case 4:eq2 = *((unsigned*)(read_buffer2+pos2)) != transp2;break;
	    default: g_warning("gglframe_overlap: Unsupported color depth\n");
	    }
	    if (eq1 && eq2)
	    {
		int x,y;
		if (dx)
		    search_border(frame2, i+area2.x,j+area2.y, dx,dy, &x, &y);
		else {x=i+area2.x;y=j+area2.y;}
		if (angle) *angle=gglframe_angle(frame2,x,y);
		return 1;
	    }
	    pos1+=bits_size1;
	    pos2+=bits_size2;
	}
	pos1+=skip1;
	pos2+=skip2;
    }
    return 0;
}

int ggl_frame_collision (GglFrame *frame1, GglFrame *frame2, int x1, int y1, int x2, int y2, float *angle)
{
    GglRectangle r1,r2;
    r1.x=MAX(0,x2-x1);
    r1.y=MAX(0,y2-y1);
    r1.width=MIN(frame1->width+x1,frame2->width+x2)-MAX(x1,x2);
    r1.height=MIN(frame1->height+y1,frame2->height+y2)-MAX(y1,y2);

    r2.x=MAX(0,x1-x2);
    r2.y=MAX(0,y1-y2);
    r2.width=r1.width;
    r2.height=r1.height;

    return gglframe_overlap(frame1,frame2,r1,r2,x1-x2,y1-y2,angle);
}
