/* GGL - GPUL Game Library
 * Copyright (C) 1998 G.P.U.L. (Group of programmers and users of Linux)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <ggl/gglgraphic.h>

void ggl_graphic_init (GglGraphic *graphic)
{
    ggl_observer_init(GGL_OBSERVER(graphic));
    graphic-> id=GGLGRAPHIC;
    graphic-> modified=1; /* The first time all graphics must be drawed */
    graphic-> visible=1;
    graphic-> x=0.0;
    graphic-> y=0.0;
    graphic-> z=0.0;
    graphic-> user_modif=0;
    graphic-> draw=ggl_do_nothing;
    graphic-> change=(void*)ggl_do_nothing;
    graphic-> get_area=ggl_do_nothing;
    graphic-> set_visible=ggl_do_nothing;
}

void ggl_graphic_finalize (GglGraphic *graphic)
{
    graphic->visible=0;
}

void ggl_graphic_draw (GglGraphic *graphic,GglVisual *visual,int ofs_x,int ofs_y, GList *overlaps)
{
    graphic->draw(graphic,visual,ofs_x,ofs_y,overlaps);
    graphic->modified=0;
}

int ggl_graphic_change (GglGraphic *graphic)
{
    return graphic->change(graphic);
}

int ggl_graphic_has_changed (GglGraphic *graphic)
{
    return graphic->modified;
}

void ggl_graphic_get_area (GglGraphic *graphic,GglRectangle *area)
{
    graphic->get_area(graphic,area);
}

void ggl_graphic_set_visible (GglGraphic *graphic,int visible)
{
    graphic->m_visible=visible;
    graphic->user_modif |= MODIF_VISIBLE;
    graphic->set_visible(graphic,visible);
}

int ggl_graphic_get_visible (GglGraphic *graphic)
{
    return graphic->visible;
}

