/* GGL - GPUL Game Library
 * Copyright (C) 1998 G.P.U.L. (Group of programmers and users of Linux)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <ggl/gglmask.h>
#include <ggi/ggi.h>
#include <string.h>
#include <stdlib.h>

/***************************************************************************/
unsigned char *create_mask(ggi_visual_t visual, ggi_pixel transp, int *size, unsigned char **lines)
/***************************************************************************/
{
    ggi_mode m;
    int width,height;
    ggi_pixel color;
    ggi_pixel seq_type;
    int seq=0;
    int i,j;
    int first=1; /* indicates first sequence */
    unsigned char *mask;
    unsigned char *last_mask;
    int mask_size=0;
    
    ggiGetMode(visual,&m);
    width=m.visible.x;
    height=m.visible.y;
    mask=(unsigned char*)malloc(width*height);
    for (j=0;j<height;j++)
    {
        ggiGetPixel(visual,0,j,&color);
        seq_type=color;
        seq=1;
        first=1;
	lines[j]=(unsigned char*)mask_size; /* We add the base pointer later */
	if (first && (color!=transp)) /* we always start with the transparent sequence */
	{
            mask[mask_size]=0;
            mask_size++;
	    first=0;
	}
        for (i=1;i<width;i++)
        {
            ggiGetPixel(visual,i,j,&color);
            if ((seq_type==transp)==(color==transp) && seq<255)
                seq++;
            else
            {
                mask[mask_size]=seq;
                mask_size++;
                if ((seq_type==transp)==(color==transp)) /*then reached the 255 limit */
                {
                    /* Then we write 0 elems of the opposite value
                       for continuing later with the same value */
                    mask[mask_size]=0;
                    mask_size++;
                }
                seq_type=color;
                seq=1;
            }
        }
        mask[mask_size]=seq; /* The last pixel/sequence isn't writed yet */
        mask_size++;
    }
    *size=mask_size;
    /* Now, we limit the memory used by the mask to mask_size */
    last_mask=(unsigned char *)malloc(mask_size);
    memcpy(last_mask,mask,mask_size);
    free(mask);

    /* Now, it's time to add the base pointer to all lines */
    for (j=0;j<height;j++) 
       lines[j]=(unsigned char*)((int)lines[j]+(int)last_mask);
    return last_mask;
    
}

/***************************************************************************/
void ggl_mask_init(GglMask *mask, GglVisual *visual, GglColor *transparent)
/***************************************************************************/
{
    mask-> id=GGLMASK;
    mask-> lines=(unsigned char **)malloc(visual->height*sizeof(unsigned char *));
    mask-> data=create_mask(visual->visual,transparent->color, &mask->size, mask->lines);
    mask-> width=visual->width;
    mask-> height=visual->height;
}

/***************************************************************************/
GglMask* ggl_mask_new(GglVisual *visual, GglColor *transparent)
/***************************************************************************/
{
    GglMask *mask=(GglMask *)malloc(sizeof(GglMask));
    ggl_base_set_size(GGL_BASE(mask),sizeof(GglMask));
    
    ggl_mask_init(mask,visual,transparent);
    
    return mask;
}

/***************************************************************************/
void ggl_mask_finalize(GglMask *mask)
/***************************************************************************/
{
  free(mask->data);
}

/***************************************************************************/
void ggl_mask_dispose(GglMask *mask)
/***************************************************************************/
{
  ggl_mask_finalize(mask);
  free(mask);
}

/***************************************************************************/
char* gglmask_search_nth(GglMask *mask, int nth, int line,int *flag, int *offs)
/***************************************************************************/
/* Finds the specified "flag" in the compressed mask, and returns a pointer to
the correspondent byte in compressed data. It returns the correspondent flag too.*/
{
  unsigned char *data=mask->data;
  int real_position,virtual_position; /* Real position is in compressed units*/
  int x_position;
  *flag=0;
  
  real_position=mask->lines[line]-data;
  virtual_position=data[real_position];
  x_position=virtual_position;
  while(virtual_position-1<nth)
    {
      real_position++;
      virtual_position+=data[real_position];
      if (x_position<mask->width)
        {
          x_position+=data[real_position];
          *flag=!*flag;
        }
      else
        {
          x_position=data[real_position];
          *flag=0;
        }
    }
*offs=data[real_position]-(virtual_position-nth);
return data+real_position;
}

#ifndef MAX
  #define MAX(a,b) (a)>(b)?(a):(b)
#endif
 
#ifndef MIN
  #define MIN(a,b) (a)<(b)?(a):(b)
#endif


/***************************************************************************/
int ggl_mask_overlap(GglMask *mask1, GglMask *mask2, GglRectangle area1, GglRectangle area2)
/***************************************************************************/
{
  int i,j,flag1,flag2,offs1,offs2;
  unsigned char *data1, *data2;

  for (j=0;j<area1.height;j++)
    {
    data1=gglmask_search_nth(mask1,area1.x,(area1.y+j),&flag1,&offs1);
    data2=gglmask_search_nth(mask2,area2.x,(area2.y+j),&flag2,&offs2);
    for (i=0;i<area1.width;)
       {
         if (flag1==1 && flag2==1) {return 1;}
         if (MIN(*data1-offs1,*data2-offs2)==*data1-offs1)
           {
             offs2+=*data1-offs1;
             i+=*data1-offs1;
             data1++;
             offs1=0;
             flag1=!flag1;
             if (*data2==offs2)
               {
                 offs2=0;
                 data2++;
                 flag2=!flag2;
               }
           }
         else
           {
             offs1+=*data2-offs2;
             i+=*data2-offs2;
             data2++;
             offs2=0;
             flag2=!flag2;
             if (*data1==offs1)
               {
                 offs1=0;
                 data1++;
                 flag1=!flag1;
               }
           }
         
       }
    }
  return 0;
}
