/* GGL - GPUL Game Library
 * Copyright (C) 1998 G.P.U.L. (Group of programmers and users of Linux)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <ggl/gglpalette.h>
#include <glib.h>
#include <stdlib.h>

void ggl_palette_init(GglPalette *palette, int first, int number)
{
    palette-> id=GGLPALETTE;
    palette-> palette=(ggi_color*)malloc(number*sizeof(ggi_color));
    palette-> first=first;
    palette-> number=number;
}

GglPalette* ggl_palette_new(int first, int number)
{
    GglPalette *palette=(GglPalette *)malloc(sizeof(GglPalette));
    ggl_base_set_size(GGL_BASE(palette),sizeof(GglPalette));

    ggl_palette_init(palette,first, number);

    return palette;
}

GglPalette* ggl_palette_new_from_array(int first, int number, char* array)
{
    GglPalette *palette=(GglPalette *)malloc(sizeof(GglPalette));
    int i;
    ggl_base_set_size(GGL_BASE(palette),sizeof(GglPalette));
    palette-> id=GGLPALETTE;
    palette-> palette=(ggi_color*)malloc(number*sizeof(ggi_color));
    palette-> first=first;
    palette-> number=number;
    for (i=0;i<number;i++)
       {
       palette->palette[i].r=array[i*3+0]<<8;
       palette->palette[i].g=array[i*3+1]<<8;
       palette->palette[i].b=array[i*3+2]<<8;
       palette->palette[i].a=0;
       }
    return palette;
}

void ggl_palette_finalize(GglPalette *palette)
{
  free(palette->palette);
}

void ggl_palette_dispose(GglPalette *palette)
{
  ggl_palette_finalize(palette);
  free(palette);
}

void ggl_palette_set_color(GglPalette *palette,int index,int r, int g, int b, int a)
{
if (index<palette->first+palette->number)
  {
  palette->palette[index].r=r;
  palette->palette[index].g=g;
  palette->palette[index].b=b;
  palette->palette[index].a=a;
  }
else g_error("ggl_palette_set_color: index out of bounds\n");
}
