/* GGL - GPUL Game Library
 * Copyright (C) 1998 G.P.U.L. (Group of programmers and users of Linux)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <ggl/gglscreen.h>

GglScreen* ggl_screen_new (int width, int height, int vwidth, int vheight,
			   GglDepth depth, char *display_type)
{
    GglScreen *screen=(GglScreen*)malloc(sizeof(GglScreen));
    ggl_base_set_size(GGL_BASE(screen),sizeof(GglScreen));

    ggl_screen_init(screen,width,height,vwidth,vheight,depth,display_type);
    return screen;
}

GglScreen* ggl_screen_new_onvisual (GglVisual *visual, int width, int height,
				    int vwidth, int vheight, GglDepth depth)
{
    GglScreen *screen=(GglScreen*)malloc(sizeof(GglScreen));
    ggl_base_set_size(GGL_BASE(screen),sizeof(GglScreen));

    ggl_screen_init_onvisual(screen,visual,width,height,vwidth,vheight,depth);
    return screen;
}

void ggl_screen_init (GglScreen *scr, int width, int height, int vwidth,
		      int vheight, GglDepth depth, char *display_type)
{
    scr-> id         = GGLSCREEN;
    scr-> timer      = NULL;
    scr-> layers     = g_list_append(NULL,ggl_layer_new());
    scr-> screen     = ggl_visual_new();

    if (scr->screen==NULL) g_error("ggl_screen_init: Cannot create visual\n");
    if (ggl_visual_check_mode(scr->screen,width,height,vwidth,vheight,depth,2)==0)
    {
        ggl_visual_set_mode(scr->screen,width,height,vwidth,vheight,depth,2);
	scr-> page_flipping=1;
        scr-> buffer=scr-> screen;
    }else{
	scr-> page_flipping=0;
	if (ggl_visual_check_mode(scr->screen,width,height,vwidth,vheight,depth,1)==0)
	{
            ggl_visual_check_mode(scr->screen,width,height,vwidth,vheight,depth,1);
	    scr-> buffer=ggl_visual_new_target("display-memory");
	    if (ggl_visual_set_mode(scr-> buffer,width,height,vwidth,vheight,depth,1)!=0)
		g_error("ggl_screen_init: Cannot create second buffer");
	}else g_error("ggl_screen_init: Cannot set mode %d x %d\n",width,height);
    }
}

void ggl_screen_init_onvisual (GglScreen *scr, GglVisual *visual, int width, int height,
				     int vwidth, int vheight, GglDepth depth)
{
    scr-> id         = GGLSCREEN;
    scr-> timer      = NULL;
    scr-> layers     = g_list_append(NULL,ggl_layer_new());
    scr-> screen     = visual;

    if (scr->screen==NULL) g_error("ggl_screen_init: Cannot create visual\n");
    if (ggl_visual_check_mode(scr->screen,width,height,vwidth,vheight,depth,2)==0)
    {
        ggl_visual_set_mode(scr->screen,width,height,vwidth,vheight,depth,2);
	scr-> page_flipping=1;
        scr-> buffer=scr-> screen;
    }else{
	scr-> page_flipping=0;
	if (ggl_visual_check_mode(scr->screen,width,height,vwidth,vheight,depth,1)==0)
	{
            ggl_visual_check_mode(scr->screen,width,height,vwidth,vheight,depth,1);
	    scr-> buffer=ggl_visual_new_target("display-memory");
	    if (ggl_visual_set_mode(scr-> buffer,width,height,vwidth,vheight,depth,1)!=0)
		g_error("ggl_screen_init: Cannot create second buffer");
	}else g_error("ggl_screen_init: Cannot set mode %d x %d\n",width,height);
    }
}

void ggl_screen_dispose	(GglScreen *screen)
{
    ggl_screen_finalize(screen);
    free(screen);
}

void ggl_screen_finalize (GglScreen *screen)
{
    GList *item;
    GglLayer *layer;
    ggl_timeout_dispose(screen->timer);
    for (item=screen->layers;item!=NULL;item=item->next)
    {
	layer=(GglLayer*)item->data;
	ggl_layer_dispose(layer);
    }
    g_list_free(screen->layers);
    ggl_visual_dispose(screen->screen);
    if (!screen->page_flipping) ggl_visual_dispose(screen->buffer);
}

void ggl_screen_add_layer (GglScreen *screen, GglLayer *layer)
{
    screen-> layers = g_list_append(screen->layers,layer);
}

void ggl_screen_remove_layer (GglScreen *screen, GglLayer *layer)
{
    screen-> layers = g_list_remove(screen->layers,layer);
}

int ggl_screen_get_layers (GglScreen *screen)
{
    return g_list_length(screen->layers);
}

GglLayer* ggl_screen_get_layer_nth(GglScreen *screen,int layer)
{
    return (GglLayer*)g_list_nth_data(screen->layers,layer);
}

int ggl_screen_get_mode	(GglScreen *screen)
{
    return screen->page_flipping;
}

GglVisual* ggl_screen_get_phys_visual(GglScreen *screen)
{
    return screen->screen;
}

GglTimeout* ggl_screen_get_timeout(GglScreen *screen)
{
    return screen->timer;
}

void gglscreen_timeout(GglScreen *screen)
{
    GList *item;
    GList *rectangles=NULL;
    for (item=screen->layers;item!=NULL;item=item->next)
    {
	GglLayer *layer=(GglLayer*)item->data;
	ggl_layer_change(layer,&rectangles);
    }

    for (item=screen->layers;item!=NULL;item=item->next)
    {
	GglLayer *layer=(GglLayer*)item->data;
	ggl_layer_draw(layer,screen->buffer,rectangles);
    }

    for (item=rectangles;item!=NULL;)
    {
	GglRectangle *rectangle=(GglRectangle*)item->data;
	item=item->next;
	ggl_visual_flush_region(screen->screen,rectangle->x,rectangle->y,
				rectangle->width,rectangle->height);
        // Freeing memory
	free(rectangle);
	rectangles=g_list_remove(rectangles,rectangle);
    }
}

void ggl_screen_show (GglScreen *screen, int fps)
{
    GglCallback cb;
    screen->timer=ggl_timeout_new(0,1000000/fps);
    cb.callback=(GglFunc)gglscreen_timeout;
    cb.data=screen;

    ggl_timeout_add_callback(screen->timer,cb);
    ggl_timeout_start(screen->timer);
}
