/* GGL - GPUL Game Library
 * Copyright (C) 1998 G.P.U.L. (Group of programmers and users of Linux)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <ggl/gglvideo.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/errno.h>

/* Constructors */
void ggl_video_init (GglVideo *video)
{
	video->id = GGLVIDEO;

	video->file = NULL;
	video->numframes = 0;
	video->currentframe = 0;

	video->h_size = 0;
	video->v_size = 0;

	video->screen = NULL;
}
/* Init all the internal structures (format dependant - forbidden) */
GglVideo * ggl_video_new (void)
{
	GglVideo *video=(GglVideo *)malloc(sizeof(GglVideo));
	ggl_base_set_size(GGL_BASE(video),sizeof(GglVideo));

	ggl_video_init(video);

	return video;
}

/*
 * File Methods (forbidden video methods - see particular format
 * implementations)
 */
void ggl_video_open (GglVideo *video, char *fname)
{
	FILE *fichero;
	
	if (NULL == (fichero = fopen(fname,"r")))
		g_print("libggl:ggl_video_open:%s:%s",fname,g_strerror(errno));
	else
		video->file = fichero;
}

void ggl_video_close (GglVideo *video)
{
	if (EOF == fclose (video->file))
		g_print("libggl:ggl_video_close::%s",g_strerror(errno));
	else
		video->file = NULL;
}

void ggl_video_attach (GglVideo *video, FILE *file_id)
{
	video->file = file_id;
}

void ggl_video_release (GglVideo *video)
{
	video->file = NULL;
}

/* Display methods */
void ggl_video_attach_screen (GglVideo *video, GglScreen *screen, int x, int y)
{
  GglRectangle rectangle;
  
  video->screen = screen;
  rectangle.x = x;
  rectangle.y = y;
  rectangle.width = x + video->h_size;
  rectangle.height = y + video->v_size;
//  ggl_screen_add_rectangle(video->screen,rectangle);
}

void ggl_video_release_screen (GglVideo *video)
{
	video->screen = NULL;
}

/* Play methods */
void ggl_video_play (GglVideo *video)
{
	ggl_do_nothing ();
}

void ggl_video_play_segment (GglVideo *video, unsigned long frame_ini,
			     unsigned long frame_fin)
{
	ggl_do_nothing ();
}

/* Frame selection methods */
void ggl_video_rewind (GglVideo *video)
{
	video->currentframe = 0;
}

void ggl_video_goto (GglVideo *video, unsigned long frame)
{
	video->currentframe = frame;
}

void ggl_video_next (GglVideo *video)
{
	(video->currentframe)++;
}

void ggl_video_previous	(GglVideo *video)
{
	(video->currentframe)--;
}

/* Single frame draw (format dependant) */
void ggl_video_draw_frame (GglVideo *video)
{
}

/* Property methods */
unsigned long ggl_video_frames (GglVideo *video)
{
	return(video->numframes);
}

char ggl_video_end (GglVideo *video)
{
	return ((video->currentframe) >= (video->numframes));
}

unsigned long ggl_video_current (GglVideo *video)
{
	return (video->currentframe);
}

int ggl_video_size_h (GglVideo *video)
{
	return (video->h_size);
}

int ggl_video_size_v (GglVideo *video)
{
	return (video->v_size);
}

