#!/usr/bin/perl

#  Copyright (C) 1999  G.P.U.L. (Group of programmers and users of Linux)
#
#  This script is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

my $DEBUG_MODE="";

my $LINE_POS=0;
my $LINE_FILE=0;

sub error
{
    my $err=shift(@_);
    print("ERROR! at line $LINE_POS of $LINE_FILE:\n\t* $err\n");
    exit(1);
}


##################################################################
#                FUNCIONES PARA GENERAR SGML                     #
##################################################################
sub sgmlheader
{
    print "<!DOCTYPE linuxdoc system>\n";
    print "<book>\n\n";
    print "<title>libGGL API Manual\n";
    print "<author>by GGL Documentation tool (ggldoc)\n";
    print "<abstract>\n";
    print "This document describes the application programming interface for libGGL.\n";
    print "</abstract>\n";
}
sub sgmltail
{
    print "\n</book>\n";
}

sub parrafo
{
    $txt=shift(@_);
    return "<p>$txt\n";
}

sub negrita
{
    $txt=shift(@_);
    return "<bf>$txt</bf>";
}

sub item
{
    $txt=shift(@_);
    return "<item>$txt\n";
}

sub truetype
{
    $txt=shift(@_);
    return "<tt>$txt</tt>";
}

sub list
{
    $txt=shift(@_);
    return "<p><enum>\n$txt</enum>\n";
}

sub Chapter
{
    $txt=shift(@_);
    return "<chapt>$txt\n";
}

sub endChapter
{
    return "</chapt>\n";
}

sub sect1
{
    $txt=shift(@_);
    return "<sect>$txt\n";
}

sub end_sect1
{
    return "</sect>\n";
}

sub sect2
{
    $txt=shift(@_);
    return "<sect1><bf>$txt</bf>\n"
}

sub end_sect2
{
    return "</sect1>\n";
}

sub sect3
{
    $txt=shift(@_);
    return "<sect2>$txt\n"
}

sub end_sect3
{
    return "</sect2>\n";
}


##################################################################
#                        PREPROCESADOR                           #
##################################################################
sub preprocesador
{
    my $fichero=shift(@_);
    local *IN;
    open(IN,$fichero);
    my $linepos=0;
    print OUT "\n";
    while(<IN>)
    {
        $linepos++;
        if  (not /^[ \t]*$/)
        {
            if (/#include "([^"]+)"/)
            {
                preprocesador($1);
            }else{
                print OUT "$linepos:$fichero:";
                print OUT;
            }
        }
    }
    
    close(IN);
}

##################################################################
#            FUNCIONES PARA PARSEAR EL FICHERO                   #
##################################################################
sub newline
{    
    $la_linea="\t";
    while ($la_linea =~ /^[ \t]*$/ && $la_linea)
    {
        $la_linea=<INPUT>;
    }
    $la_linea=~ /^([^:]+):([^:]+):(.*)$/;
    $LINE_POS=$1;
    $LINE_FILE=$2;
    $la_linea=$3;
    return $la_linea;
}

sub definicion
{
    if ($linea =~ /^[ \t]*%=[ \t]*(.*)/)
    {
        if ($DEBUG_MODE) {print("($LINE_POS)");}
        $linea = $1 || newline();
        if ($DEBUG_MODE) {print("[%=]\n");}
    }else{
        return;
    }
    return "%=";
}

sub texto
{
    my $txt="";
    while ($linea =~ /^[ \t]*([^%]+)(.*)/)
    {
        if ($DEBUG_MODE) {print("($LINE_POS)");}
        $linea = $2 || newline();
        if ($txt) {$txt="$txt $1";}
        else {$txt=$1;}
    }
    if ($DEBUG_MODE) {print("[txt] $txt\n");}
    return $txt;
}

sub palabra_reservada
{
    my $palabra=shift(@_);
    
    if ($linea =~ /^[ \t]*$palabra[ \t]*(.*)/)
    {
        if ($DEBUG_MODE) {print("($LINE_POS)");}
        $linea = $1 || newline();
        if ($DEBUG_MODE) {print("[$palabra] ($linea)\n");}
    }else{
        return
    }
    return $palabra;
}

sub palabra
{
    my $palabra="";
    if ($linea =~ /^[ \t]*([^%]+)(.*)/)
    {
        if ($DEBUG_MODE) {print("($LINE_POS)");}
        $linea = $2 || newline();
        $palabra=$1;
        if ($DEBUG_MODE) {print("[word] $palabra\n");}
    }else{
        return;
    }
    return $palabra;
}

$module_="%[mM][oO][dD][uU][lL][eE]";
$class_="%[cC][lL][aA][sS][sS]";
$method_="%[mM][eE][tT][hH][oO][dD]";
$format_="%[fF][oO][rR][mM][aA][tT]";
$father_="%[fF][aA][tT][hH][eE][rR]";
$arglist_="%[aA][rR][gG][lL][iI][sS][tT]";
$arg_="%[aA][rR][gG]";
$return_="%[rR][eE][tT][uU][rR][nN]";

sub methods
{
    while (palabra_reservada("$method_"))
    {
        my $name=palabra() || error("METHOD_NAME");
        definicion() || error("%= missing");
        my $coment=texto();

        print "\n";
        print sect2($name);
        print parrafo($coment);
        palabra_reservada("$format_") || error("%format missing");

        my $format=texto();

        print "\n";
        print sect3("Format:");
        print parrafo(truetype($format));
        print end_sect3();

        print "\n";
        print sect3("Arguments");
        if (palabra_reservada($arglist_))
        {
            my $argline="";
            while (palabra_reservada($arg_))
            {
                my $argname=palabra() || error ("arg. name missing");
                definicion() || error("%= missing");
                my $argdesc=texto();
                $argadd=item("$argname: $argdesc");
                $argline="$argline\t$argadd";
            }
            print list($argline);
        }else{
            print parrafo("This method has no arguments\n");
        }
        print end_sect3();
        if  (palabra_reservada($return_))
        {
            if ($DEBUG_MODE) {print("[%return]\n");}
            my $desc=texto();
            print sect3("Return");
            print parrafo($desc);
            print end_sect3;
        }
        print end_sect2();
    }
}

sub classes
{    
    while (palabra_reservada($class_))
    {
        my $name=palabra() || error ("class name missing");
        definicion() || error ("%= missing");
        my $desc=texto();
        palabra_reservada($father_) || error ("%father missing");
        my $father=palabra() || error ("father name missing");
        my $output=negrita("Father:");
        print sect1("Class $name");
        print parrafo("$output $father");
        print parrafo($desc);
        print parrafo("This class provides us the following methods:");
        methods();
        print end_sect1();
    }
}

sub modules
{
    while (palabra_reservada($module_))
    {
        my $name=palabra() || error ("module name missing");
        definicion() || error ("%= missing");
        my $desc=texto();
        print Chapter($name);
        print parrafo($desc);
        classes();
        print endChapter();
    }
}

##################################################################
#                     CONTROL PARAMETROS                         #
##################################################################
if ($#ARGV<0)
{
    print "GGLDOC v1.1 - GGL DOCUMENTATION UTILITY -\n";
    print "Sintax: ggldoc.pl <filename>\n";
    print "\t* The sgml documentation is given to the standard output\n\n";
    exit(1);
}

sgmlheader();
while (@ARGV)
{
    my $fichero=shift @ARGV;

    my $TMPFILE="/tmp/_%_%%$$%%_%_";
    open(OUT,">$TMPFILE") || error "can't create temporal file.";

    preprocesador($fichero);
    close(OUT);
    open(INPUT,"<$TMPFILE");


    $linea=newline();
    modules();
    close(INPUT);
    unlink($TMPFILE);
}
sgmltail();
 